@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 13958 2013-02-04 18:42:52Z jordan $
+ * $Id: main.c 14112 2013-07-08 19:34:58Z jordan $
  *
  * Copyright (c) Transmission authors and contributors
  *
@@ -88,7 +88,6 @@ struct cbdata
   GSList * duplicates_list;
   GSList * details;
   GtkTreeSelection * sel;
-  gpointer quit_dialog;
 };
 
 static void
@@ -161,7 +160,7 @@ show_details_dialog_for_selected_torrents (struct cbdata * data)
   char * key = get_details_dialog_key (ids);
 
   for (l=data->details; dialog==NULL && l!=NULL; l=l->next)
-    if (!strcmp (key, g_object_get_data (l->data, "key")))
+    if (!g_strcmp0 (key, g_object_get_data (l->data, "key")))
       dialog = l->data;
 
   if (dialog == NULL)
@@ -233,6 +232,7 @@ static gboolean
 refresh_actions (gpointer gdata)
 {
   struct cbdata * data = gdata;
+
   if (!data->is_closing)
     {
       int canUpdate;
@@ -293,22 +293,43 @@ on_selection_changed (GtkTreeSelection * s UNUSED, gpointer gdata)
 ****
 ***/
 
+static gboolean
+has_magnet_link_handler (void)
+{
+  GAppInfo * app_info = g_app_info_get_default_for_uri_scheme ("magnet");
+  const gboolean has_handler = app_info != NULL;
+  g_clear_object (&app_info);
+  return has_handler;
+}
+
 static void
 register_magnet_link_handler (void)
 {
-  GAppInfo * app_info = g_app_info_get_default_for_uri_scheme ("magnet");
-  if (app_info == NULL)
+  GError * error;
+  GAppInfo * app;
+  const char * const content_type = "x-scheme-handler/magnet";
+
+  error = NULL;
+  app = g_app_info_create_from_commandline ("transmission-gtk",
+                                            "transmission-gtk",
+                                            G_APP_INFO_CREATE_SUPPORTS_URIS,
+                                            &error);
+  g_app_info_set_as_default_for_type (app, content_type, &error);
+  if (error != NULL)
     {
-      /* there's no default magnet handler, so register ourselves for the job... */
-      GError * error = NULL;
-      app_info = g_app_info_create_from_commandline ("transmission-gtk", "transmission-gtk", G_APP_INFO_CREATE_SUPPORTS_URIS, NULL);
-      g_app_info_set_as_default_for_type (app_info, "x-scheme-handler/magnet", &error);
-      if (error != NULL)
-        {
-          g_warning (_("Error registering Transmission as x-scheme-handler/magnet handler: %s"), error->message);
-          g_clear_error (&error);
-        }
+      g_warning (_("Error registering Transmission as a %s handler: %s"),
+                 content_type,
+                 error->message);
+      g_error_free (error);
     }
+
+  g_clear_object (&app);
+}
+static void
+ensure_magnet_handler_exists (void)
+{
+  if (!has_magnet_link_handler ())
+    register_magnet_link_handler ();
 }
 
 static void
@@ -320,17 +341,17 @@ on_main_window_size_allocated (GtkWidget      * gtk_window,
   const gboolean isMaximized = (gdk_window != NULL)
                             && (gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_MAXIMIZED);
 
-  gtr_pref_int_set (PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED, isMaximized);
+  gtr_pref_int_set (TR_KEY_main_window_is_maximized, isMaximized);
 
   if (!isMaximized)
     {
       int x, y, w, h;
       gtk_window_get_position (GTK_WINDOW (gtk_window), &x, &y);
       gtk_window_get_size (GTK_WINDOW (gtk_window), &w, &h);
-      gtr_pref_int_set (PREF_KEY_MAIN_WINDOW_X, x);
-      gtr_pref_int_set (PREF_KEY_MAIN_WINDOW_Y, y);
-      gtr_pref_int_set (PREF_KEY_MAIN_WINDOW_WIDTH, w);
-      gtr_pref_int_set (PREF_KEY_MAIN_WINDOW_HEIGHT, h);
+      gtr_pref_int_set (TR_KEY_main_window_x, x);
+      gtr_pref_int_set (TR_KEY_main_window_y, y);
+      gtr_pref_int_set (TR_KEY_main_window_width, w);
+      gtr_pref_int_set (TR_KEY_main_window_height, h);
     }
 }
 
@@ -338,109 +359,76 @@ on_main_window_size_allocated (GtkWidget      * gtk_window,
 **** listen to changes that come from RPC
 ***/
 
-struct rpc_idle_data
+struct on_rpc_changed_struct
 {
   TrCore * core;
-  int id;
-  gboolean delete_files;
+  tr_rpc_callback_type type;
+  int torrent_id;
 };
 
 static gboolean
-rpc_torrent_remove_idle (gpointer gdata)
-{
-  struct rpc_idle_data * data = gdata;
-
-  gtr_core_remove_torrent (data->core, data->id, data->delete_files);
-
-  g_free (data);
-  return FALSE; /* tell g_idle not to call this func twice */
-}
-
-static gboolean
-rpc_torrent_add_idle (gpointer gdata)
+on_rpc_changed_idle (gpointer gdata)
 {
   tr_torrent * tor;
-  struct rpc_idle_data * data = gdata;
+  struct on_rpc_changed_struct * data = gdata;
 
-  if ((tor = gtr_core_find_torrent (data->core, data->id)))
-    gtr_core_add_torrent (data->core, tor, TRUE);
-
-  g_free (data);
-  return FALSE; /* tell g_idle not to call this func twice */
-}
-
-static tr_rpc_callback_status
-on_rpc_changed (tr_session            * session,
-                tr_rpc_callback_type    type,
-                struct tr_torrent     * tor,
-                void                  * gdata)
-{
-  tr_rpc_callback_status status = TR_RPC_OK;
-  struct cbdata * cbdata = gdata;
-  gdk_threads_enter ();
-
-  switch (type)
+  switch (data->type)
     {
       case TR_RPC_SESSION_CLOSE:
         gtr_action_activate ("quit");
         break;
 
-      case TR_RPC_TORRENT_ADDED: {
-        struct rpc_idle_data * data = g_new0 (struct rpc_idle_data, 1);
-        data->id = tr_torrentId (tor);
-        data->core = cbdata->core;
-        gdk_threads_add_idle (rpc_torrent_add_idle, data);
+      case TR_RPC_TORRENT_ADDED:
+        if ((tor = gtr_core_find_torrent (data->core, data->torrent_id)))
+          gtr_core_add_torrent (data->core, tor, true);
         break;
-      }
 
       case TR_RPC_TORRENT_REMOVING:
-      case TR_RPC_TORRENT_TRASHING: {
-        struct rpc_idle_data * data = g_new0 (struct rpc_idle_data, 1);
-        data->id = tr_torrentId (tor);
-        data->core = cbdata->core;
-        data->delete_files = type == TR_RPC_TORRENT_TRASHING;
-        gdk_threads_add_idle (rpc_torrent_remove_idle, data);
-        status = TR_RPC_NOREMOVE;
+        gtr_core_remove_torrent (data->core, data->torrent_id, false);
+        break;
+
+      case TR_RPC_TORRENT_TRASHING:
+        gtr_core_remove_torrent (data->core, data->torrent_id, true);
         break;
-      }
 
       case TR_RPC_SESSION_CHANGED: {
         int i;
-        tr_benc tmp;
-        tr_benc * newval;
-        tr_benc * oldvals = gtr_pref_get_all ();
-        const char * key;
+        tr_variant tmp;
+        tr_variant * newval;
+        tr_variant * oldvals = gtr_pref_get_all ();
+        tr_quark key;
         GSList * l;
         GSList * changed_keys = NULL;
-        tr_bencInitDict (&tmp, 100);
+        tr_session * session = gtr_core_session (data->core);
+        tr_variantInitDict (&tmp, 100);
         tr_sessionGetSettings (session, &tmp);
-        for (i=0; tr_bencDictChild (&tmp, i, &key, &newval); ++i)
+        for (i=0; tr_variantDictChild (&tmp, i, &key, &newval); ++i)
           {
             bool changed;
-            tr_benc * oldval = tr_bencDictFind (oldvals, key);
+            tr_variant * oldval = tr_variantDictFind (oldvals, key);
             if (!oldval)
               {
                 changed = true;
               }
             else
               {
-                char * a = tr_bencToStr (oldval, TR_FMT_BENC, NULL);
-                char * b = tr_bencToStr (newval, TR_FMT_BENC, NULL);
-                changed = strcmp (a, b) != 0;
+                char * a = tr_variantToStr (oldval, TR_VARIANT_FMT_BENC, NULL);
+                char * b = tr_variantToStr (newval, TR_VARIANT_FMT_BENC, NULL);
+                changed = g_strcmp0 (a, b) != 0;
                 tr_free (b);
                 tr_free (a);
               }
 
             if (changed)
-              changed_keys = g_slist_append (changed_keys, (gpointer)key);
+              changed_keys = g_slist_append (changed_keys, GINT_TO_POINTER(key));
           }
         tr_sessionGetSettings (session, oldvals);
 
         for (l=changed_keys; l!=NULL; l=l->next)
-          gtr_core_pref_changed (cbdata->core, l->data);
+          gtr_core_pref_changed (data->core, GPOINTER_TO_INT(l->data));
 
         g_slist_free (changed_keys);
-        tr_bencFree (&tmp);
+        tr_variantFree (&tmp);
         break;
       }
 
@@ -453,8 +441,26 @@ on_rpc_changed (tr_session            * session,
         break;
     }
 
-  gdk_threads_leave ();
-  return status;
+  g_free (data);
+  return G_SOURCE_REMOVE;
+}
+
+static tr_rpc_callback_status
+on_rpc_changed (tr_session            * session G_GNUC_UNUSED,
+                tr_rpc_callback_type    type,
+                struct tr_torrent     * tor,
+                void                  * gdata)
+{
+  struct cbdata * cbdata = gdata;
+  struct on_rpc_changed_struct * data;
+
+  data = g_new (struct on_rpc_changed_struct, 1);
+  data->core = cbdata->core;
+  data->type = type;
+  data->torrent_id = tr_torrentId (tor);
+  gdk_threads_add_idle (on_rpc_changed_idle, data);
+  
+  return TR_RPC_NOREMOVE;
 }
 
 /***
@@ -502,16 +508,16 @@ on_startup (GApplication * application, gpointer user_data)
   sighandler_cbdata = cbdata;
 
   /* ensure the directories are created */
-  if ((str = gtr_pref_string_get (TR_PREFS_KEY_DOWNLOAD_DIR)))
+  if ((str = gtr_pref_string_get (TR_KEY_download_dir)))
     g_mkdir_with_parents (str, 0777);
-  if ((str = gtr_pref_string_get (TR_PREFS_KEY_INCOMPLETE_DIR)))
+  if ((str = gtr_pref_string_get (TR_KEY_incomplete_dir)))
     g_mkdir_with_parents (str, 0777);
 
   /* initialize the libtransmission session */
   session = tr_sessionInit ("gtk", cbdata->config_dir, TRUE, gtr_pref_get_all ());
 
-  gtr_pref_flag_set (TR_PREFS_KEY_ALT_SPEED_ENABLED, tr_sessionUsesAltSpeed (session));
-  gtr_pref_int_set (TR_PREFS_KEY_PEER_PORT, tr_sessionGetPeerPort (session));
+  gtr_pref_flag_set (TR_KEY_alt_speed_enabled, tr_sessionUsesAltSpeed (session));
+  gtr_pref_int_set  (TR_KEY_peer_port, tr_sessionGetPeerPort (session));
   cbdata->core = gtr_core_new (session);
 
   /* init the ui manager */
@@ -531,11 +537,11 @@ on_startup (GApplication * application, gpointer user_data)
   tr_sessionSetRPCCallback (session, on_rpc_changed, cbdata);
 
   /* check & see if it's time to update the blocklist */
-  if (gtr_pref_flag_get (TR_PREFS_KEY_BLOCKLIST_ENABLED))
+  if (gtr_pref_flag_get (TR_KEY_blocklist_enabled))
     {
-      if (gtr_pref_flag_get (PREF_KEY_BLOCKLIST_UPDATES_ENABLED))
+      if (gtr_pref_flag_get (TR_KEY_blocklist_updates_enabled))
         {
-          const int64_t last_time = gtr_pref_int_get ("blocklist-date");
+          const int64_t last_time = gtr_pref_int_get (TR_KEY_blocklist_date);
           const int SECONDS_IN_A_WEEK = 7 * 24 * 60 * 60;
           const time_t now = time (NULL);
           if (last_time + SECONDS_IN_A_WEEK < now)
@@ -544,7 +550,7 @@ on_startup (GApplication * application, gpointer user_data)
     }
 
   /* if there's no magnet link handler registered, register us */
-  register_magnet_link_handler ();
+  ensure_magnet_handler_exists ();
 }
 
 static void
@@ -565,8 +571,8 @@ static void
 open_files (GSList * files, gpointer gdata)
 {
   struct cbdata * cbdata = gdata;
-  const gboolean do_start = gtr_pref_flag_get (TR_PREFS_KEY_START) && !cbdata->start_paused;
-  const gboolean do_prompt = gtr_pref_flag_get (PREF_KEY_OPTIONS_PROMPT);
+  const gboolean do_start = gtr_pref_flag_get (TR_KEY_start_added_torrents) && !cbdata->start_paused;
+  const gboolean do_prompt = gtr_pref_flag_get (TR_KEY_show_options_window);
   const gboolean do_notify = TRUE;
 
   gtr_core_add_files (cbdata->core, files, do_start, do_prompt, do_notify);
@@ -625,7 +631,9 @@ main (int argc, char ** argv)
   textdomain (MY_READABLE_NAME);
 
   /* init glib/gtk */
+#if !GLIB_CHECK_VERSION(2,35,4)
   g_type_init ();
+#endif
   gtk_init (&argc, &argv);
   g_set_application_name (_("Transmission"));
   gtk_window_set_default_icon_name (MY_CONFIG_NAME);
@@ -683,7 +691,7 @@ on_core_busy (TrCore * core UNUSED, gboolean busy, struct cbdata * c)
 
 static void on_core_error (TrCore *, guint, const char *, struct cbdata *);
 static void on_add_torrent (TrCore *, tr_ctor *, gpointer);
-static void on_prefs_changed (TrCore * core, const char * key, gpointer);
+static void on_prefs_changed (TrCore * core, const tr_quark key, gpointer);
 static void main_window_setup (struct cbdata * cbdata, GtkWindow * wind);
 static gboolean update_model_loop (gpointer gdata);
 static gboolean update_model_once (gpointer gdata);
@@ -692,7 +700,7 @@ static void
 app_setup (GtkWindow * wind, struct cbdata * cbdata)
 {
   if (cbdata->is_iconified)
-    gtr_pref_flag_set (PREF_KEY_SHOW_TRAY_ICON, TRUE);
+    gtr_pref_flag_set (TR_KEY_show_notification_area_icon, TRUE);
 
   gtr_actions_set_core (cbdata->core);
 
@@ -710,7 +718,7 @@ app_setup (GtkWindow * wind, struct cbdata * cbdata)
   main_window_setup (cbdata, wind);
 
   /* set up the icon */
-  on_prefs_changed (cbdata->core, PREF_KEY_SHOW_TRAY_ICON, cbdata);
+  on_prefs_changed (cbdata->core, TR_KEY_show_notification_area_icon, cbdata);
 
   /* start model update timer */
   cbdata->timer = gdk_threads_add_timeout_seconds (MAIN_WINDOW_REFRESH_INTERVAL_SECONDS, update_model_loop, cbdata);
@@ -729,22 +737,22 @@ app_setup (GtkWindow * wind, struct cbdata * cbdata)
       gtr_action_set_toggled ("toggle-main-window", FALSE);
     }
 
-  if (!gtr_pref_flag_get (PREF_KEY_USER_HAS_GIVEN_INFORMED_CONSENT))
+  if (!gtr_pref_flag_get (TR_KEY_user_has_given_informed_consent))
     {
       GtkWidget * w = gtk_message_dialog_new (GTK_WINDOW (wind),
                                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                                              GTK_MESSAGE_INFO,
+                                              GTK_MESSAGE_OTHER,
                                               GTK_BUTTONS_NONE,
                                               "%s",
-        _("Transmission is a file-sharing program. When you run a torrent, its data will be made available to others by means of upload. You and you alone are fully responsible for exercising proper judgement and abiding by your local laws."));
-      gtk_dialog_add_button (GTK_DIALOG (w), GTK_STOCK_QUIT, GTK_RESPONSE_REJECT);
-      gtk_dialog_add_button (GTK_DIALOG (w), _("I _Accept"), GTK_RESPONSE_ACCEPT);
+        _("Transmission is a file sharing program. When you run a torrent, its data will be made available to others by means of upload. Any content you share is your sole responsibility."));
+      gtk_dialog_add_button (GTK_DIALOG (w), GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT);
+      gtk_dialog_add_button (GTK_DIALOG (w), _("I _Agree"), GTK_RESPONSE_ACCEPT);
       gtk_dialog_set_default_response (GTK_DIALOG (w), GTK_RESPONSE_ACCEPT);
       switch (gtk_dialog_run (GTK_DIALOG (w)))
         {
           case GTK_RESPONSE_ACCEPT:
             /* only show it once */
-            gtr_pref_flag_set (PREF_KEY_USER_HAS_GIVEN_INFORMED_CONSENT, TRUE);
+            gtr_pref_flag_set (TR_KEY_user_has_given_informed_consent, TRUE);
             gtk_widget_destroy (w);
             break;
 
@@ -768,10 +776,10 @@ presentMainWindow (struct cbdata * cbdata)
 
   if (!gtk_widget_get_visible (GTK_WIDGET (window)))
     {
-      gtk_window_resize (window, gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_WIDTH),
-                                 gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_HEIGHT));
-      gtk_window_move (window, gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_X),
-                               gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_Y));
+      gtk_window_resize (window, gtr_pref_int_get (TR_KEY_main_window_width),
+                                 gtr_pref_int_get (TR_KEY_main_window_height));
+      gtk_window_move (window, gtr_pref_int_get (TR_KEY_main_window_x),
+                               gtr_pref_int_get (TR_KEY_main_window_y));
       gtr_widget_set_visible (GTK_WIDGET (window), TRUE);
     }
 
@@ -901,20 +909,25 @@ on_session_closed (gpointer gdata)
   g_slist_foreach (cbdata->duplicates_list, (GFunc)g_free, NULL);
   g_slist_free (cbdata->duplicates_list);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
+struct session_close_struct
+{
+  tr_session * session;
+  struct cbdata * cbdata;
+};
+
+/* since tr_sessionClose () is a blocking function,
+ * delegate its call to another thread here... when it's done,
+ * punt the GUI teardown back to the GTK+ thread */
 static gpointer
 session_close_threadfunc (gpointer gdata)
 {
-  /* since tr_sessionClose () is a blocking function,
-   * call it from another thread... when it's done,
-   * punt the GUI teardown back to the GTK+ thread */
-  struct cbdata * cbdata = gdata;
-  gdk_threads_enter ();
-  gtr_core_close (cbdata->core);
-  gdk_threads_add_idle (on_session_closed, gdata);
-  gdk_threads_leave ();
+  struct session_close_struct * data = gdata;
+  tr_sessionClose (data->session);
+  gdk_threads_add_idle (on_session_closed, data->cbdata);
+  g_free (data);
   return NULL;
 }
 
@@ -928,7 +941,8 @@ static void
 on_app_exit (gpointer vdata)
 {
   GtkWidget *r, *p, *b, *w, *c;
-  struct cbdata *cbdata = vdata;
+  struct cbdata * cbdata = vdata;
+  struct session_close_struct * session_close_data;
 
   cbdata->is_closing = true;
 
@@ -983,13 +997,16 @@ on_app_exit (gpointer vdata)
   /* ensure the window is in its previous position & size.
    * this seems to be necessary because changing the main window's
    * child seems to unset the size */
-  gtk_window_resize (cbdata->wind, gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_WIDTH),
-                                   gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_HEIGHT));
-  gtk_window_move (cbdata->wind, gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_X),
-                                 gtr_pref_int_get (PREF_KEY_MAIN_WINDOW_Y));
+  gtk_window_resize (cbdata->wind, gtr_pref_int_get (TR_KEY_main_window_width),
+                                   gtr_pref_int_get (TR_KEY_main_window_height));
+  gtk_window_move (cbdata->wind, gtr_pref_int_get (TR_KEY_main_window_x),
+                                 gtr_pref_int_get (TR_KEY_main_window_y));
 
   /* shut down libT */
-  g_thread_new ("shutdown-thread", session_close_threadfunc, vdata);
+  session_close_data = g_new (struct session_close_struct, 1);
+  session_close_data->cbdata = cbdata;
+  session_close_data->session = gtr_core_close (cbdata->core);
+  g_thread_new ("shutdown-thread", session_close_threadfunc, session_close_data);
 }
 
 static void
@@ -1091,192 +1108,201 @@ on_add_torrent (TrCore * core, tr_ctor * ctor, gpointer gdata)
 }
 
 static void
-on_prefs_changed (TrCore * core UNUSED, const char * key, gpointer data)
+on_prefs_changed (TrCore * core UNUSED, const tr_quark key, gpointer data)
 {
   struct cbdata * cbdata = data;
   tr_session * tr = gtr_core_session (cbdata->core);
 
-  if (!strcmp (key, TR_PREFS_KEY_ENCRYPTION))
-    {
-      tr_sessionSetEncryption (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_DOWNLOAD_DIR))
-    {
-      tr_sessionSetDownloadDir (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_MSGLEVEL))
-    {
-      tr_setMessageLevel (gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_PEER_PORT))
-    {
-      tr_sessionSetPeerPort (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_BLOCKLIST_ENABLED))
+  switch (key)
     {
+      case TR_KEY_encryption:
+        tr_sessionSetEncryption (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_download_dir:
+        tr_sessionSetDownloadDir (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_message_level:
+        tr_logSetLevel (gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_peer_port:
+        tr_sessionSetPeerPort (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_blocklist_enabled:
         tr_blocklistSetEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_BLOCKLIST_URL))
-    {
+        break;
+
+      case TR_KEY_blocklist_url:
         tr_blocklistSetURL (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, PREF_KEY_SHOW_TRAY_ICON))
-    {
-      const int show = gtr_pref_flag_get (key);
-      if (show && !cbdata->icon)
-        cbdata->icon = gtr_icon_new (cbdata->core);
-      else if (!show && cbdata->icon)
-        g_clear_object (&cbdata->icon);
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_DSPEED_ENABLED))
-    {
-      tr_sessionLimitSpeed (tr, TR_DOWN, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_DSPEED_KBps))
-    {
-      tr_sessionSetSpeedLimit_KBps (tr, TR_DOWN, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_USPEED_ENABLED))
-    {
-      tr_sessionLimitSpeed (tr, TR_UP, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_USPEED_KBps))
-    {
-      tr_sessionSetSpeedLimit_KBps (tr, TR_UP, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RATIO_ENABLED))
-    {
-      tr_sessionSetRatioLimited (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RATIO))
-    {
-      tr_sessionSetRatioLimit (tr, gtr_pref_double_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_IDLE_LIMIT))
-    {
-      tr_sessionSetIdleLimit (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_IDLE_LIMIT_ENABLED))
-    {
-      tr_sessionSetIdleLimited (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_PORT_FORWARDING))
-    {
-      tr_sessionSetPortForwardingEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_PEX_ENABLED))
-    {
-      tr_sessionSetPexEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RENAME_PARTIAL_FILES))
-    {
-      tr_sessionSetIncompleteFileNamingEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_DOWNLOAD_QUEUE_SIZE))
-    {
-      tr_sessionSetQueueSize (tr, TR_DOWN, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_QUEUE_STALLED_MINUTES))
-    {
-      tr_sessionSetQueueStalledMinutes (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_DHT_ENABLED))
-    {
-      tr_sessionSetDHTEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_UTP_ENABLED))
-    {
-      tr_sessionSetUTPEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_LPD_ENABLED))
-    {
-      tr_sessionSetLPDEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_PORT))
-    {
-      tr_sessionSetRPCPort (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_ENABLED))
-    {
-      tr_sessionSetRPCEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_WHITELIST))
-    {
-      tr_sessionSetRPCWhitelist (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_WHITELIST_ENABLED))
-    {
-      tr_sessionSetRPCWhitelistEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_USERNAME))
-    {
-      tr_sessionSetRPCUsername (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_PASSWORD))
-    {
-      tr_sessionSetRPCPassword (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_RPC_AUTH_REQUIRED))
-    {
-      tr_sessionSetRPCPasswordEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_UP_KBps))
-    {
-      tr_sessionSetAltSpeed_KBps (tr, TR_UP, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_DOWN_KBps))
-    {
+        break;
+
+      case TR_KEY_show_notification_area_icon:
+        {
+          const bool show = gtr_pref_flag_get (key);
+          if (show && !cbdata->icon)
+            cbdata->icon = gtr_icon_new (cbdata->core);
+          else if (!show && cbdata->icon)
+            g_clear_object (&cbdata->icon);
+          break;
+        }
+
+      case TR_KEY_speed_limit_down_enabled:
+        tr_sessionLimitSpeed (tr, TR_DOWN, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_speed_limit_down:
+        tr_sessionSetSpeedLimit_KBps (tr, TR_DOWN, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_speed_limit_up_enabled:
+        tr_sessionLimitSpeed (tr, TR_UP, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_speed_limit_up:
+        tr_sessionSetSpeedLimit_KBps (tr, TR_UP, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_ratio_limit_enabled:
+        tr_sessionSetRatioLimited (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_ratio_limit:
+        tr_sessionSetRatioLimit (tr, gtr_pref_double_get (key));
+        break;
+
+      case TR_KEY_idle_seeding_limit:
+        tr_sessionSetIdleLimit (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_idle_seeding_limit_enabled:
+        tr_sessionSetIdleLimited (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_port_forwarding_enabled:
+        tr_sessionSetPortForwardingEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_pex_enabled:
+        tr_sessionSetPexEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_rename_partial_files:
+        tr_sessionSetIncompleteFileNamingEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_download_queue_size:
+        tr_sessionSetQueueSize (tr, TR_DOWN, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_queue_stalled_minutes:
+        tr_sessionSetQueueStalledMinutes (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_dht_enabled:
+        tr_sessionSetDHTEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_utp_enabled:
+        tr_sessionSetUTPEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_lpd_enabled:
+        tr_sessionSetLPDEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_rpc_port:
+        tr_sessionSetRPCPort (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_rpc_enabled:
+        tr_sessionSetRPCEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_rpc_whitelist:
+        tr_sessionSetRPCWhitelist (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_rpc_whitelist_enabled:
+        tr_sessionSetRPCWhitelistEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_rpc_username:
+        tr_sessionSetRPCUsername (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_rpc_password:
+        tr_sessionSetRPCPassword (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_rpc_authentication_required:
+        tr_sessionSetRPCPasswordEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_alt_speed_up:
+        tr_sessionSetAltSpeed_KBps (tr, TR_UP, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_alt_speed_down:
         tr_sessionSetAltSpeed_KBps (tr, TR_DOWN, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_ENABLED))
-    {
-      const gboolean b = gtr_pref_flag_get (key);
-      tr_sessionUseAltSpeed (tr, b);
-      gtr_action_set_toggled (key, b);
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_TIME_BEGIN))
-    {
-      tr_sessionSetAltSpeedBegin (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_TIME_END))
-    {
-      tr_sessionSetAltSpeedEnd (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED))
-    {
-      tr_sessionUseAltSpeedTime (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_ALT_SPEED_TIME_DAY))
-    {
-      tr_sessionSetAltSpeedDay (tr, gtr_pref_int_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START))
-    {
-      tr_sessionSetPeerPortRandomOnStart (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_INCOMPLETE_DIR))
-    {
-      tr_sessionSetIncompleteDir (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED))
-    {
-      tr_sessionSetIncompleteDirEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED))
-    {
-      tr_sessionSetTorrentDoneScriptEnabled (tr, gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME))
-    {
-      tr_sessionSetTorrentDoneScript (tr, gtr_pref_string_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_START))
-    {
-      tr_sessionSetPaused (tr, !gtr_pref_flag_get (key));
-    }
-  else if (!strcmp (key, TR_PREFS_KEY_TRASH_ORIGINAL))
-    {
-      tr_sessionSetDeleteSource (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_alt_speed_enabled:
+        {
+          const bool b = gtr_pref_flag_get (key);
+          tr_sessionUseAltSpeed (tr, b);
+          gtr_action_set_toggled (tr_quark_get_string(key,NULL), b);
+          break;
+        }
+
+      case TR_KEY_alt_speed_time_begin:
+        tr_sessionSetAltSpeedBegin (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_alt_speed_time_end:
+        tr_sessionSetAltSpeedEnd (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_alt_speed_time_enabled:
+        tr_sessionUseAltSpeedTime (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_alt_speed_time_day:
+        tr_sessionSetAltSpeedDay (tr, gtr_pref_int_get (key));
+        break;
+
+      case TR_KEY_peer_port_random_on_start:
+        tr_sessionSetPeerPortRandomOnStart (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_incomplete_dir:
+        tr_sessionSetIncompleteDir (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_incomplete_dir_enabled:
+        tr_sessionSetIncompleteDirEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_script_torrent_done_enabled:
+        tr_sessionSetTorrentDoneScriptEnabled (tr, gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_script_torrent_done_filename:
+        tr_sessionSetTorrentDoneScript (tr, gtr_pref_string_get (key));
+        break;
+
+      case TR_KEY_start_added_torrents:
+        tr_sessionSetPaused (tr, !gtr_pref_flag_get (key));
+        break;
+
+      case TR_KEY_trash_original_torrent_files:
+        tr_sessionSetDeleteSource (tr, gtr_pref_flag_get (key));
+        break;
+
+      default:
+        break;
     }
 }
 
@@ -1300,7 +1326,7 @@ update_model_once (gpointer gdata)
     gtr_icon_refresh (data->icon);
 
   data->update_model_soon_tag = 0;
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -1357,33 +1383,33 @@ append_id_to_benc_list (GtkTreeModel * m, GtkTreePath * path UNUSED,
 {
   tr_torrent * tor = NULL;
   gtk_tree_model_get (m, iter, MC_TORRENT, &tor, -1);
-  tr_bencListAddInt (list, tr_torrentId (tor));
+  tr_variantListAddInt (list, tr_torrentId (tor));
 }
 
 static gboolean
 call_rpc_for_selected_torrents (struct cbdata * data, const char * method)
 {
-  tr_benc top, *args, *ids;
+  tr_variant top, *args, *ids;
   gboolean invoked = FALSE;
   GtkTreeSelection * s = data->sel;
   tr_session * session = gtr_core_session (data->core);
 
-  tr_bencInitDict (&top, 2);
-  tr_bencDictAddStr (&top, "method", method);
-  args = tr_bencDictAddDict (&top, "arguments", 1);
-  ids = tr_bencDictAddList (args, "ids", 0);
+  tr_variantInitDict (&top, 2);
+  tr_variantDictAddStr (&top, TR_KEY_method, method);
+  args = tr_variantDictAddDict (&top, TR_KEY_arguments, 1);
+  ids = tr_variantDictAddList (args, TR_KEY_ids, 0);
   gtk_tree_selection_selected_foreach (s, append_id_to_benc_list, ids);
 
-  if (tr_bencListSize (ids) != 0)
+  if (tr_variantListSize (ids) != 0)
     {
       int json_len;
-      char * json = tr_bencToStr (&top, TR_FMT_JSON_LEAN, &json_len);
+      char * json = tr_variantToStr (&top, TR_VARIANT_FMT_JSON_LEAN, &json_len);
       tr_rpc_request_exec_json (session, json, json_len, NULL, NULL);
       g_free (json);
       invoked = TRUE;
     }
 
-  tr_bencFree (&top);
+  tr_variantFree (&top);
   return invoked;
 }
 
@@ -1487,35 +1513,35 @@ gtr_actions_handler (const char * action_name, gpointer user_data)
   gboolean changed = FALSE;
   struct cbdata * data = user_data;
 
-  if (!strcmp (action_name, "open-torrent-from-url"))
+  if (!g_strcmp0 (action_name, "open-torrent-from-url"))
     {
       GtkWidget * w = gtr_torrent_open_from_url_dialog_new (data->wind, data->core);
       gtk_widget_show (w);
     }
-  else if (!strcmp (action_name, "open-torrent-menu")
-        || !strcmp (action_name, "open-torrent-toolbar"))
+  else if (!g_strcmp0 (action_name, "open-torrent-menu")
+        || !g_strcmp0 (action_name, "open-torrent-toolbar"))
     {
       GtkWidget * w = gtr_torrent_open_from_file_dialog_new (data->wind, data->core);
       gtk_widget_show (w);
     }
-    else if (!strcmp (action_name, "show-stats"))
+    else if (!g_strcmp0 (action_name, "show-stats"))
     {
         GtkWidget * dialog = gtr_stats_dialog_new (data->wind, data->core);
         gtk_widget_show (dialog);
     }
-  else if (!strcmp (action_name, "donate"))
+  else if (!g_strcmp0 (action_name, "donate"))
     {
       gtr_open_uri ("http://www.transmissionbt.com/donate.php");
     }
-  else if (!strcmp (action_name, "pause-all-torrents"))
+  else if (!g_strcmp0 (action_name, "pause-all-torrents"))
     {
       pause_all_torrents (data);
     }
-  else if (!strcmp (action_name, "start-all-torrents"))
+  else if (!g_strcmp0 (action_name, "start-all-torrents"))
     {
       start_all_torrents (data);
     }
-  else if (!strcmp (action_name, "copy-magnet-link-to-clipboard"))
+  else if (!g_strcmp0 (action_name, "copy-magnet-link-to-clipboard"))
     {
       tr_torrent * tor = get_first_selected_torrent (data);
       if (tor != NULL)
@@ -1523,7 +1549,7 @@ gtr_actions_handler (const char * action_name, gpointer user_data)
           copy_magnet_link_to_clipboard (GTK_WIDGET (data->wind), tor);
         }
     }
-  else if (!strcmp (action_name, "relocate-torrent"))
+  else if (!g_strcmp0 (action_name, "relocate-torrent"))
     {
       GSList * ids = get_selected_torrent_ids (data);
       if (ids != NULL)
@@ -1533,52 +1559,52 @@ gtr_actions_handler (const char * action_name, gpointer user_data)
           gtk_widget_show (w);
         }
     }
-  else if (!strcmp (action_name, "torrent-start")
-        || !strcmp (action_name, "torrent-start-now")
-        || !strcmp (action_name, "torrent-stop")
-        || !strcmp (action_name, "torrent-reannounce")
-        || !strcmp (action_name, "torrent-verify")
-        || !strcmp (action_name, "queue-move-top")
-        || !strcmp (action_name, "queue-move-up")
-        || !strcmp (action_name, "queue-move-down")
-        || !strcmp (action_name, "queue-move-bottom"))
+  else if (!g_strcmp0 (action_name, "torrent-start")
+        || !g_strcmp0 (action_name, "torrent-start-now")
+        || !g_strcmp0 (action_name, "torrent-stop")
+        || !g_strcmp0 (action_name, "torrent-reannounce")
+        || !g_strcmp0 (action_name, "torrent-verify")
+        || !g_strcmp0 (action_name, "queue-move-top")
+        || !g_strcmp0 (action_name, "queue-move-up")
+        || !g_strcmp0 (action_name, "queue-move-down")
+        || !g_strcmp0 (action_name, "queue-move-bottom"))
     {
       changed |= call_rpc_for_selected_torrents (data, action_name);
     }
-  else if (!strcmp (action_name, "open-torrent-folder"))
+  else if (!g_strcmp0 (action_name, "open-torrent-folder"))
     {
       gtk_tree_selection_selected_foreach (data->sel, open_folder_foreach, data->core);
     }
-  else if (!strcmp (action_name, "show-torrent-properties"))
+  else if (!g_strcmp0 (action_name, "show-torrent-properties"))
     {
       show_details_dialog_for_selected_torrents (data);
     }
-  else if (!strcmp (action_name, "new-torrent"))
+  else if (!g_strcmp0 (action_name, "new-torrent"))
     {
       GtkWidget * w = gtr_torrent_creation_dialog_new (data->wind, data->core);
       gtk_widget_show (w);
     }
-  else if (!strcmp (action_name, "remove-torrent"))
+  else if (!g_strcmp0 (action_name, "remove-torrent"))
     {
       remove_selected (data, FALSE);
     }
-  else if (!strcmp (action_name, "delete-torrent"))
+  else if (!g_strcmp0 (action_name, "delete-torrent"))
     {
       remove_selected (data, TRUE);
     }
-  else if (!strcmp (action_name, "quit"))
+  else if (!g_strcmp0 (action_name, "quit"))
     {
       on_app_exit (data);
     }
-  else if (!strcmp (action_name, "select-all"))
+  else if (!g_strcmp0 (action_name, "select-all"))
     {
       gtk_tree_selection_select_all (data->sel);
     }
-  else if (!strcmp (action_name, "deselect-all"))
+  else if (!g_strcmp0 (action_name, "deselect-all"))
     {
       gtk_tree_selection_unselect_all (data->sel);
     }
-  else if (!strcmp (action_name, "edit-preferences"))
+  else if (!g_strcmp0 (action_name, "edit-preferences"))
     {
       if (NULL == data->prefs)
         {
@@ -1588,7 +1614,7 @@ gtr_actions_handler (const char * action_name, gpointer user_data)
         }
         gtr_window_present (GTK_WINDOW (data->prefs));
     }
-  else if (!strcmp (action_name, "toggle-message-log"))
+  else if (!g_strcmp0 (action_name, "toggle-message-log"))
     {
       if (!data->msgwin)
         {
@@ -1603,19 +1629,19 @@ gtr_actions_handler (const char * action_name, gpointer user_data)
           data->msgwin = NULL;
         }
     }
-  else if (!strcmp (action_name, "show-about-dialog"))
+  else if (!g_strcmp0 (action_name, "show-about-dialog"))
     {
       show_about_dialog (data->wind);
     }
-  else if (!strcmp (action_name, "help"))
+  else if (!g_strcmp0 (action_name, "help"))
     {
       gtr_open_uri (gtr_get_help_uri ());
     }
-  else if (!strcmp (action_name, "toggle-main-window"))
+  else if (!g_strcmp0 (action_name, "toggle-main-window"))
     {
       toggleMainWindow (data);
     }
-  else if (!strcmp (action_name, "present-main-window"))
+  else if (!g_strcmp0 (action_name, "present-main-window"))
     {
       presentMainWindow (data);
     }
