@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: open-dialog.c 13625 2012-12-05 17:29:46Z jordan $
+ * $Id: open-dialog.c 14077 2013-05-22 20:35:38Z jordan $
  */
 
 #include <glib/gi18n.h>
@@ -42,7 +42,7 @@ get_recent_destinations (void)
         char key[64];
         const char * val;
         g_snprintf (key, sizeof (key), "recent-download-dir-%d", i+1);
-        if ((val = gtr_pref_string_get (key)))
+        if ((val = gtr_pref_string_get (tr_quark_new(key,-1))))
             list = g_slist_append (list, (void*)val);
     }
     return list;
@@ -59,7 +59,7 @@ save_recent_destination (TrCore * core, const char * dir)
         return;
 
     /* if it was already in the list, remove it */
-    if ((l = g_slist_find_custom (list, dir, (GCompareFunc)strcmp)))
+    if ((l = g_slist_find_custom (list, dir, (GCompareFunc)g_strcmp0)))
         list = g_slist_delete_link (list, l);
 
     /* add it to the front of the list */
@@ -74,7 +74,7 @@ save_recent_destination (TrCore * core, const char * dir)
     for (l=list, i=0; l && (i<N_RECENT); ++i, l=l->next) {
         char key[64];
         g_snprintf (key, sizeof (key), "recent-download-dir-%d", i + 1);
-        gtr_pref_string_set (key, l->data);
+        gtr_pref_string_set (tr_quark_new(key,-1), l->data);
     }
     gtr_pref_save (gtr_core_session (core));
 
@@ -94,6 +94,7 @@ struct OpenData
     GtkWidget *  run_check;
     GtkWidget *  trash_check;
     GtkWidget *  priority_combo;
+    GtkWidget *  freespace_label;
     char *       filename;
     char *       downloadDir;
     tr_torrent * tor;
@@ -163,7 +164,7 @@ updateTorrent (struct OpenData * o)
         tr_torrentSetDownloadDir (o->tor, o->downloadDir);
         gtk_widget_set_sensitive (o->file_list, tr_torrentHasMetadata (o->tor));
         gtr_file_list_set_torrent (o->file_list, tr_torrentId (o->tor));
-        tr_torrentVerify (o->tor);
+        tr_torrentVerify (o->tor, NULL, NULL);
     }
 }
 
@@ -185,6 +186,7 @@ sourceChanged (GtkFileChooserButton * b, gpointer gdata)
     {
         int err = 0;
         int new_file = 0;
+        int duplicate_id = 0;
         tr_torrent * torrent;
 
         if (filename && (!o->filename || !tr_is_same_file (filename, o->filename)))
@@ -199,14 +201,21 @@ sourceChanged (GtkFileChooserButton * b, gpointer gdata)
         tr_ctorSetPaused (o->ctor, TR_FORCE, TRUE);
         tr_ctorSetDeleteSource (o->ctor, FALSE);
 
-        if ((torrent = tr_torrentNew (o->ctor, &err)))
+        if ((torrent = tr_torrentNew (o->ctor, &err, &duplicate_id)))
         {
             removeOldTorrent (o);
             o->tor = torrent;
         }
         else if (new_file)
         {
-            gtr_add_torrent_error_dialog (GTK_WIDGET (b), err, o->filename);
+            tr_torrent * tor;
+
+            if (duplicate_id)
+              tor = gtr_core_find_torrent (o->core, duplicate_id);
+            else
+              tor = NULL;
+
+            gtr_add_torrent_error_dialog (GTK_WIDGET (b), err, tor, o->filename);
         }
 
         updateTorrent (o);
@@ -226,6 +235,8 @@ downloadDirChanged (GtkFileChooserButton * b, gpointer gdata)
         g_free (data->downloadDir);
         data->downloadDir = g_strdup (fname);
         updateTorrent (data);
+
+        gtr_freespace_label_set_dir (data->freespace_label, data->downloadDir);
     }
 
     g_free (fname);
@@ -339,6 +350,13 @@ gtr_torrent_options_dialog_new (GtkWindow * parent, TrCore * core, tr_ctor * cto
     g_signal_connect (w, "selection-changed",
                       G_CALLBACK (downloadDirChanged), data);
 
+    row++;
+    l = data->freespace_label = gtr_freespace_label_new (core, data->downloadDir);
+    gtk_widget_set_margin_bottom (l, GUI_PAD_BIG);
+    gtk_misc_set_alignment (GTK_MISC (l), 1.0f, 0.5f);
+    gtk_grid_attach (grid, l, 0, row, 2, 1);
+
+
     // file list row
     row++;
     w = data->file_list;
@@ -396,7 +414,7 @@ onOpenDialogResponse (GtkDialog * dialog, int response, gpointer core)
 
     /* remember this folder the next time we use this dialog */
     folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
-    gtr_pref_string_set (PREF_KEY_OPEN_DIALOG_FOLDER, folder);
+    gtr_pref_string_set (TR_KEY_open_dialog_dir, folder);
     g_free (folder);
 
     if (response == GTK_RESPONSE_ACCEPT)
@@ -404,7 +422,7 @@ onOpenDialogResponse (GtkDialog * dialog, int response, gpointer core)
         GtkFileChooser  * chooser = GTK_FILE_CHOOSER (dialog);
         GtkWidget       * w = gtk_file_chooser_get_extra_widget (chooser);
         GtkToggleButton * tb = GTK_TOGGLE_BUTTON (w);
-        const gboolean    do_start = gtr_pref_flag_get (TR_PREFS_KEY_START);
+        const gboolean    do_start = gtr_pref_flag_get (TR_KEY_start_added_torrents);
         const gboolean    do_prompt = gtk_toggle_button_get_active (tb);
         const gboolean    do_notify = FALSE;
         GSList * files = gtk_file_chooser_get_files (chooser);
@@ -437,12 +455,12 @@ gtr_torrent_open_from_file_dialog_new (GtkWindow * parent, TrCore * core)
     addTorrentFilters (GTK_FILE_CHOOSER (w));
     g_signal_connect (w, "response", G_CALLBACK (onOpenDialogResponse), core);
 
-    if ((folder = gtr_pref_string_get (PREF_KEY_OPEN_DIALOG_FOLDER)))
+    if ((folder = gtr_pref_string_get (TR_KEY_open_dialog_dir)))
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (w), folder);
 
     c = gtk_check_button_new_with_mnemonic (_("Show _options dialog"));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (c),
-                                 gtr_pref_flag_get (PREF_KEY_OPTIONS_PROMPT));
+                                 gtr_pref_flag_get (TR_KEY_show_options_window));
     gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (w), c);
     gtk_widget_show (c);
 
