@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: announcer.c 13631 2012-12-07 01:53:31Z jordan $
+ * $Id: announcer.c 14124 2013-07-16 00:13:30Z jordan $
  */
 
 #include <assert.h>
@@ -25,6 +25,7 @@
 #include "announcer.h"
 #include "announcer-common.h"
 #include "crypto.h" /* tr_cryptoRandInt (), tr_cryptoWeakRandInt () */
+#include "log.h"
 #include "peer-mgr.h" /* tr_peerMgrCompactToPex () */
 #include "ptrarray.h"
 #include "session.h"
@@ -37,11 +38,16 @@ static void tier_build_log_name (const struct tr_tier * tier,
                                  char * buf, size_t buflen);
 
 #define dbgmsg(tier, ...) \
-if (tr_deepLoggingIsActive ()) do { \
-  char name[128]; \
-  tier_build_log_name (tier, name, sizeof (name)); \
-  tr_deepLog (__FILE__, __LINE__, name, __VA_ARGS__); \
-} while (0)
+  do \
+    { \
+      if (tr_logGetDeepEnabled ()) \
+        { \
+          char name[128]; \
+          tier_build_log_name (tier, name, sizeof (name)); \
+          tr_logAddDeep (__FILE__, __LINE__, name, __VA_ARGS__); \
+        } \
+    } \
+  while (0)
 
 enum
 {
@@ -749,7 +755,7 @@ tr_announcerNextManualAnnounce (const tr_torrent * tor)
 static void
 dbgmsg_tier_announce_queue (const tr_tier * tier)
 {
-    if (tr_deepLoggingIsActive ())
+    if (tr_logGetDeepEnabled ())
     {
         int i;
         char name[128];
@@ -765,7 +771,7 @@ dbgmsg_tier_announce_queue (const tr_tier * tier)
         }
 
         message = evbuffer_free_to_str (buf);
-        tr_deepLog (__FILE__, __LINE__, name, "announce queue is %s", message);
+        tr_logAddDeep (__FILE__, __LINE__, name, "announce queue is %s", message);
         tr_free (message);
     }
 }
@@ -899,7 +905,7 @@ tr_announcerAddBytes (tr_torrent * tor, int type, uint32_t byteCount)
 
 static tr_announce_request *
 announce_request_new (const tr_announcer  * announcer,
-                      const tr_torrent    * tor,
+                      tr_torrent          * tor,
                       const tr_tier       * tier,
                       tr_announce_event     event)
 {
@@ -908,7 +914,7 @@ announce_request_new (const tr_announcer  * announcer,
     req->url = tr_strdup (tier->currentTracker->announce);
     req->tracker_id_str = tr_strdup (tier->currentTracker->tracker_id_str);
     memcpy (req->info_hash, tor->info.hash, SHA_DIGEST_LENGTH);
-    memcpy (req->peer_id, tor->peer_id, PEER_ID_LEN);
+    memcpy (req->peer_id, tr_torrentGetPeerId(tor), PEER_ID_LEN);
     req->up = tier->byteCounts[TR_ANN_UP];
     req->down = tier->byteCounts[TR_ANN_DOWN];
     req->corrupt = tier->byteCounts[TR_ANN_CORRUPT];
@@ -956,17 +962,15 @@ tr_announcerRemoveTorrent (tr_announcer * announcer, tr_torrent * tor)
 static int
 getRetryInterval (const tr_tracker * t)
 {
-    int minutes;
-    const unsigned int jitter_seconds = tr_cryptoWeakRandInt (60);
-    switch (t->consecutiveFailures) {
-        case 0:  minutes =   1; break;
-        case 1:  minutes =   5; break;
-        case 2:  minutes =  15; break;
-        case 3:  minutes =  30; break;
-        case 4:  minutes =  60; break;
-        default: minutes = 120; break;
+  switch (t->consecutiveFailures)
+    {
+      case 0:  return 20;
+      case 1:  return tr_cryptoWeakRandInt (60) + (60 * 5);
+      case 2:  return tr_cryptoWeakRandInt (60) + (60 * 15);
+      case 3:  return tr_cryptoWeakRandInt (60) + (60 * 30);
+      case 4:  return tr_cryptoWeakRandInt (60) + (60 * 60);
+      default: return tr_cryptoWeakRandInt (60) + (60 * 120);
     }
-    return (minutes * 60) + jitter_seconds;
 }
 
 struct announce_data
@@ -991,7 +995,7 @@ on_announce_error (tr_tier * tier, const char * err, tr_announce_event e)
 
     /* set the error message */
     dbgmsg (tier, "%s", err);
-    tr_torinf (tier->tor, "%s", err);
+    tr_logAddTorInfo (tier->tor, "%s", err);
     tr_strlcpy (tier->lastAnnounceStr, err, sizeof (tier->lastAnnounceStr));
 
     /* switch to the next tracker */
@@ -1000,7 +1004,7 @@ on_announce_error (tr_tier * tier, const char * err, tr_announce_event e)
     /* schedule a reannounce */
     interval = getRetryInterval (tier->currentTracker);
     dbgmsg (tier, "Retrying announce in %d seconds.", interval);
-    tr_torinf (tier->tor, "Retrying announce in %d seconds.", interval);
+    tr_logAddTorInfo (tier->tor, "Retrying announce in %d seconds.", interval);
     tier_announce_event_push (tier, e, tr_time () + interval);
 }
 
@@ -1145,7 +1149,7 @@ on_announce_done (const tr_announce_response  * response,
                then a separate scrape isn't needed */
             if ((scrape_fields >= 3) || (!tracker->scrape && (scrape_fields >= 1)))
             {
-                tr_tordbg (tier->tor, "Announce response contained scrape info; "
+                tr_logAddTorDbg (tier->tor, "Announce response contained scrape info; "
                                       "rescheduling next scrape to %d seconds from now.",
                                       tier->scrapeIntervalSec);
                 tier->scrapeAt = get_next_scrape_time (announcer->session, tier, tier->scrapeIntervalSec);
@@ -1200,12 +1204,30 @@ announce_request_delegate (tr_announcer               * announcer,
 {
     tr_session * session = announcer->session;
 
+#if 0
+    fprintf (stderr, "ANNOUNCE: event %s isPartialSeed %d port %d key %d numwant %d"
+                     " up %"PRIu64" down %"PRIu64" corrupt %"PRIu64" left %"PRIu64
+                     " url [%s] tracker_id_str [%s] peer_id [%20.20s]\n",
+             tr_announce_event_get_string (request->event),
+             (int)request->partial_seed,
+             (int)request->port,
+             request->key,
+             request->numwant,
+             request->up,
+             request->down,
+             request->corrupt,
+             request->leftUntilComplete,
+             request->url,
+             request->tracker_id_str,
+             request->peer_id);
+#endif
+
     if (!memcmp (request->url, "http", 4))
         tr_tracker_http_announce (session, request, callback, callback_data);
     else if (!memcmp (request->url, "udp://", 6))
         tr_tracker_udp_announce (session, request, callback, callback_data);
     else
-        tr_err ("Unsupported url: %s", request->url);
+        tr_logAddError ("Unsupported url: %s", request->url);
 
     announce_request_free (request);
 }
@@ -1216,7 +1238,7 @@ tierAnnounce (tr_announcer * announcer, tr_tier * tier)
     tr_announce_event announce_event;
     tr_announce_request * req;
     struct announce_data * data;
-    const tr_torrent * tor = tier->tor;
+    tr_torrent * tor = tier->tor;
     const time_t now = tr_time ();
 
     assert (!tier->isAnnouncing);
@@ -1256,7 +1278,7 @@ on_scrape_error (tr_session * session, tr_tier * tier, const char * errmsg)
 
     /* set the error message */
     dbgmsg (tier, "Scrape error: %s", errmsg);
-    tr_torinf (tier->tor, "Scrape error: %s", errmsg);
+    tr_logAddTorInfo (tier->tor, "Scrape error: %s", errmsg);
     tr_strlcpy (tier->lastScrapeStr, errmsg, sizeof (tier->lastScrapeStr));
 
     /* switch to the next tracker */
@@ -1265,7 +1287,7 @@ on_scrape_error (tr_session * session, tr_tier * tier, const char * errmsg)
     /* schedule a rescrape */
     interval = getRetryInterval (tier->currentTracker);
     dbgmsg (tier, "Retrying scrape in %zu seconds.", (size_t)interval);
-    tr_torinf (tier->tor, "Retrying scrape in %zu seconds.", (size_t)interval);
+    tr_logAddTorInfo (tier->tor, "Retrying scrape in %zu seconds.", (size_t)interval);
     tier->lastScrapeSucceeded = false;
     tier->scrapeAt = get_next_scrape_time (session, tier, interval);
 }
@@ -1348,7 +1370,7 @@ on_scrape_done (const tr_scrape_response * response, void * vsession)
                     tier->scrapeIntervalSec = MAX (DEFAULT_SCRAPE_INTERVAL_SEC,
                                                    response->min_request_interval);
                     tier->scrapeAt = get_next_scrape_time (session, tier, tier->scrapeIntervalSec);
-                    tr_tordbg (tier->tor, "Scrape successful. Rescraping in %d seconds.",
+                    tr_logAddTorDbg (tier->tor, "Scrape successful. Rescraping in %d seconds.",
                                tier->scrapeIntervalSec);
 
                     if ((tracker = tier->currentTracker))
@@ -1384,7 +1406,7 @@ scrape_request_delegate (tr_announcer             * announcer,
     else if (!memcmp (request->url, "udp://", 6))
         tr_tracker_udp_scrape (session, request, callback, callback_data);
     else
-        tr_err ("Unsupported url: %s", request->url);
+        tr_logAddError ("Unsupported url: %s", request->url);
 }
 
 static void
@@ -1529,7 +1551,7 @@ announceMore (tr_announcer * announcer)
     n = MIN (tr_ptrArraySize (&announceMe), announcer->slotsAvailable);
     for (i=0; i<n; ++i) {
         tr_tier * tier = tr_ptrArrayNth (&announceMe, i);
-        tr_tordbg (tier->tor, "%s", "Announcing to tracker");
+        tr_logAddTorDbg (tier->tor, "%s", "Announcing to tracker");
         dbgmsg (tier, "announcing tier %d of %d", i, n);
         tierAnnounce (announcer, tier);
     }
@@ -1585,7 +1607,6 @@ tr_announcerStats (const tr_torrent * torrent, int * setmeTrackerCount)
     const time_t now = tr_time ();
 
     assert (tr_isTorrent (torrent));
-    assert (tr_torrentIsLocked (torrent));
 
     tt = torrent->tiers;
 
