@@ -1,31 +1,33 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: prefs-dialog.cc 8763 2009-06-30 18:08:50Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: prefs-dialog.cc 13991 2013-02-09 04:05:03Z jordan $
  */
 
 #include <cassert>
 #include <iostream>
 
-#include <QCoreApplication>
 #include <QCheckBox>
-#include <QFileDialog>
 #include <QComboBox>
+#include <QCoreApplication>
 #include <QDialogButtonBox>
 #include <QDoubleSpinBox>
+#include <QFileIconProvider>
+#include <QFileInfo>
 #include <QHBoxLayout>
+#include <QHttp>
+#include <QIcon>
 #include <QLabel>
 #include <QLineEdit>
-#include <QFileInfo>
 #include <QList>
+#include <QMessageBox>
 #include <QPushButton>
-#include <QIcon>
 #include <QSpinBox>
 #include <QStyle>
 #include <QTabWidget>
@@ -33,14 +35,14 @@
 #include <QTimeEdit>
 #include <QTimer>
 #include <QVBoxLayout>
-#include <QHttp>
-#include <QMessageBox>
 
+#include "freespace-label.h"
+#include "formatter.h"
 #include "hig.h"
 #include "prefs.h"
 #include "prefs-dialog.h"
-#include "qticonloader.h"
 #include "session.h"
+#include "utils.h"
 
 /***
 ****
@@ -55,7 +57,7 @@ void
 PrefsDialog :: checkBoxToggled( bool checked )
 {
     const int key( sender( )->property( PREF_KEY ).toInt( ) );
-    myPrefs.set( key, checked );
+    setPref( key, checked );
 }
 
 QCheckBox *
@@ -77,38 +79,15 @@ PrefsDialog :: enableBuddyWhenChecked( QCheckBox * box, QWidget * buddy )
 }
 
 void
-PrefsDialog :: spinBoxChangedIdle( )
+PrefsDialog :: spinBoxEditingFinished()
 {
-    const QObject * spin( sender()->property( "SPIN" ).value<QObject*>( ) );
-    const int key( spin->property( PREF_KEY ).toInt( ) );
-
+    const QObject * spin = sender();
+    const int key = spin->property( PREF_KEY ).toInt( );
     const QDoubleSpinBox * d = qobject_cast<const QDoubleSpinBox*>( spin );
-    if( d != 0 )
-        myPrefs.set( key, d->value( ) );
+    if( d )
+        setPref( key, d->value( ) );
     else
-        myPrefs.set( key, qobject_cast<const QSpinBox*>(spin)->value( ) );
-}
-
-void
-PrefsDialog :: spinBoxChanged( int value )
-{
-    Q_UNUSED( value );
-
-    static const QString timerName( "TIMER_CHILD" );
-    QObject * o( sender( ) );
-
-    // user may be spinning through many values, so let's hold off
-    // for a moment to kekep from flooding a bunch of prefs changes
-    QTimer * timer( o->findChild<QTimer*>( timerName ) );
-    if( timer == 0 )
-    {
-        timer = new QTimer( o );
-        timer->setObjectName( timerName );
-        timer->setSingleShot( true );
-        timer->setProperty( "SPIN", qVariantFromValue( o ) );
-        connect( timer, SIGNAL(timeout()), this, SLOT(spinBoxChangedIdle()));
-    }
-    timer->start( 200 );
+        setPref( key, qobject_cast<const QSpinBox*>(spin)->value( ) );
 }
 
 QSpinBox *
@@ -119,19 +98,11 @@ PrefsDialog :: spinBoxNew( int key, int low, int high, int step )
     spin->setSingleStep( step );
     spin->setValue( myPrefs.getInt( key ) );
     spin->setProperty( PREF_KEY, key );
-    connect( spin, SIGNAL(valueChanged(int)), this, SLOT(spinBoxChanged(int)));
+    connect( spin, SIGNAL(editingFinished()), this, SLOT(spinBoxEditingFinished()));
     myWidgets.insert( key, spin );
     return spin;
 }
 
-void
-PrefsDialog :: doubleSpinBoxChanged( double value )
-{
-    Q_UNUSED( value );
-
-    spinBoxChanged( 0 );
-}
-
 QDoubleSpinBox *
 PrefsDialog :: doubleSpinBoxNew( int key, double low, double high, double step, int decimals )
 {
@@ -141,39 +112,48 @@ PrefsDialog :: doubleSpinBoxNew( int key, double low, double high, double step,
     spin->setDecimals( decimals );
     spin->setValue( myPrefs.getDouble( key ) );
     spin->setProperty( PREF_KEY, key );
-    connect( spin, SIGNAL(valueChanged(double)), this, SLOT(doubleSpinBoxChanged(double)));
+    connect( spin, SIGNAL(editingFinished()), this, SLOT(spinBoxEditingFinished()));
     myWidgets.insert( key, spin );
     return spin;
 }
 
 void
-PrefsDialog :: timeChanged( const QTime& time )
+PrefsDialog :: timeEditingFinished( )
 {
-    const int key( sender()->property( PREF_KEY ).toInt( ) );
-    const int seconds( QTime().secsTo( time ) );
-    myPrefs.set( key, seconds / 60 );
+    QTimeEdit * e = qobject_cast<QTimeEdit*>(sender());
+    if( e )
+    {
+        const int key( e->property( PREF_KEY ).toInt( ) );
+        const QTime time( e->time( ) );
+        const int seconds( QTime().secsTo( time ) );
+std::cerr << "setPref to " << (seconds/60) << " minutes" << std::endl;
+        setPref( key, seconds / 60 );
+    }
 }
-
 QTimeEdit*
 PrefsDialog :: timeEditNew( int key )
 {
     const int minutes( myPrefs.getInt( key ) );
     QTimeEdit * e = new QTimeEdit( );
-    e->setDisplayFormat( "hh:mm" );
+    e->setDisplayFormat( QString::fromAscii( "hh:mm" ) );
     e->setProperty( PREF_KEY, key );
     e->setTime( QTime().addSecs( minutes * 60 ) );
     myWidgets.insert( key, e );
-    connect( e, SIGNAL(timeChanged(const QTime&)), this, SLOT(timeChanged(const QTime&)) );
+    connect( e, SIGNAL(editingFinished()), this, SLOT(timeEditingFinished()) );
     return e;
 }
 
 void
-PrefsDialog :: textChanged( const QString& text )
+PrefsDialog :: lineEditingFinished( )
 {
-    const int key( sender()->property( PREF_KEY ).toInt( ) );
-    myPrefs.set( key, text );
+    QLineEdit * e = qobject_cast<QLineEdit*>(sender());
+    if( e && e->isModified( ) )
+    {
+        const int key( e->property( PREF_KEY ).toInt( ) );
+        const QString text( e->text() );
+        setPref( key, text );
+    }
 }
-
 QLineEdit*
 PrefsDialog :: lineEditNew( int key, int echoMode )
 {
@@ -181,7 +161,7 @@ PrefsDialog :: lineEditNew( int key, int echoMode )
     e->setProperty( PREF_KEY, key );
     e->setEchoMode( QLineEdit::EchoMode( echoMode ) );
     myWidgets.insert( key, e );
-    connect( e, SIGNAL(textChanged(const QString&)), this, SLOT(textChanged(const QString&)) );
+    connect( e, SIGNAL(editingFinished()), this, SLOT(lineEditingFinished()) );
     return e;
 }
 
@@ -190,47 +170,19 @@ PrefsDialog :: lineEditNew( int key, int echoMode )
 ***/
 
 QWidget *
-PrefsDialog :: createTrackerTab( )
-{
-    QWidget *l, *r;
-    HIG * hig = new HIG( );
-    hig->addSectionTitle( tr( "Tracker Proxy" ) );
-    hig->addWideControl( l = checkBoxNew( tr( "Connect to tracker via a pro&xy" ), Prefs::PROXY_ENABLED ) );
-    myUnsupportedWhenRemote << l;
-    l = hig->addRow( tr( "Proxy &server:" ), r = lineEditNew( Prefs::PROXY ) );
-    myProxyWidgets << l << r;
-    l = hig->addRow( tr( "Proxy &port:" ), r = spinBoxNew( Prefs::PROXY_PORT, 1, 65535, 1 ) );
-    myProxyWidgets << l << r;
-    hig->addWideControl( l = checkBoxNew( tr( "Use &authentication" ), Prefs::PROXY_AUTH_ENABLED ) );
-    myProxyWidgets << l;
-    l = hig->addRow( tr( "&Username:" ), r = lineEditNew( Prefs::PROXY_USERNAME ) );
-    myProxyAuthWidgets << l << r;
-    l = hig->addRow( tr( "Pass&word:" ), r = lineEditNew( Prefs::PROXY_PASSWORD, QLineEdit::Password ) );
-    myProxyAuthWidgets << l << r;
-    myUnsupportedWhenRemote << myProxyAuthWidgets;
-    hig->finish( );
-    return hig;
-}
-
-/***
-****
-***/
-
-QWidget *
-PrefsDialog :: createWebTab( Session& session )
+PrefsDialog :: createRemoteTab( Session& session )
 {
     HIG * hig = new HIG( this );
-    hig->addSectionTitle( tr( "Web Client" ) );
+    hig->addSectionTitle( tr( "Remote Control" ) );
     QWidget * w;
     QHBoxLayout * h = new QHBoxLayout( );
-    QIcon i( style()->standardIcon( QStyle::StandardPixmap( QStyle::SP_DirOpenIcon ) ) );
-    QPushButton * b = new QPushButton( i, tr( "&Open web client" ) );
+    QPushButton * b = new QPushButton( tr( "&Open web client" ) );
     connect( b, SIGNAL(clicked()), &session, SLOT(launchWebInterface()) );
     h->addWidget( b, 0, Qt::AlignRight );
-    QWidget * l = checkBoxNew( tr( "&Enable web client" ), Prefs::RPC_ENABLED );
+    QWidget * l = checkBoxNew( tr( "Allow &remote access" ), Prefs::RPC_ENABLED );
     myUnsupportedWhenRemote << l;
     hig->addRow( l, h, 0 );
-    l = hig->addRow( tr( "Listening &port:" ), w = spinBoxNew( Prefs::RPC_PORT, 0, 65535, 1 ) );
+    l = hig->addRow( tr( "HTTP &port:" ), w = spinBoxNew( Prefs::RPC_PORT, 0, 65535, 1 ) );
     myWebWidgets << l << w;
     hig->addWideControl( w = checkBoxNew( tr( "Use &authentication" ), Prefs::RPC_AUTH_REQUIRED ) );
     myWebWidgets << w;
@@ -238,7 +190,7 @@ PrefsDialog :: createWebTab( Session& session )
     myWebAuthWidgets << l << w;
     l = hig->addRow( tr( "Pass&word:" ), w = lineEditNew( Prefs::RPC_PASSWORD, QLineEdit::Password ) );
     myWebAuthWidgets << l << w;
-    hig->addWideControl( w = checkBoxNew( tr( "Only allow these IP a&ddresses to connect:" ), Prefs::RPC_WHITELIST_ENABLED ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Only allow these IP a&ddresses:" ), Prefs::RPC_WHITELIST_ENABLED ) );
     myWebWidgets << w;
     l = hig->addRow( tr( "Addresses:" ), w = lineEditNew( Prefs::RPC_WHITELIST ) );
     myWebWhitelistWidgets << l << w;
@@ -255,7 +207,7 @@ void
 PrefsDialog :: altSpeedDaysEdited( int i )
 {
     const int value = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
-    myPrefs.set( Prefs::ALT_SPEED_LIMIT_TIME_DAY, value );
+    setPref( Prefs::ALT_SPEED_LIMIT_TIME_DAY, value );
 }
 
 
@@ -265,14 +217,15 @@ PrefsDialog :: createSpeedTab( )
     QWidget *l, *r;
     HIG * hig = new HIG( this );
     hig->addSectionTitle( tr( "Speed Limits" ) );
+    const QString speed_K_str = Formatter::unitStr( Formatter::SPEED, Formatter::KB );
 
-        l = checkBoxNew( tr( "Limit &download speed (KB/s):" ), Prefs::DSPEED_ENABLED );
-        r = spinBoxNew( Prefs::DSPEED, 0, INT_MAX, 5 );
+        l = checkBoxNew( tr( "&Upload (%1):" ).arg( speed_K_str ), Prefs::USPEED_ENABLED );
+        r = spinBoxNew( Prefs::USPEED, 0, INT_MAX, 5 );
         hig->addRow( l, r );
         enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
 
-        l = checkBoxNew( tr( "Limit &upload speed (KB/s):" ), Prefs::USPEED_ENABLED );
-        r = spinBoxNew( Prefs::USPEED, 0, INT_MAX, 5 );
+        l = checkBoxNew( tr( "&Download (%1):" ).arg( speed_K_str ), Prefs::DSPEED_ENABLED );
+        r = spinBoxNew( Prefs::DSPEED, 0, INT_MAX, 5 );
         hig->addRow( l, r );
         enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
 
@@ -280,11 +233,11 @@ PrefsDialog :: createSpeedTab( )
     QHBoxLayout * h = new QHBoxLayout;
     h->setSpacing( HIG :: PAD );
     QLabel * label = new QLabel;
-    label->setPixmap( QPixmap( ":/icons/alt-limit-off.png" ) );
+    label->setPixmap( QPixmap( QString::fromAscii( ":/icons/alt-limit-off.png" ) ) );
     label->setAlignment( Qt::AlignLeft|Qt::AlignVCenter );
     h->addWidget( label );
-    label = new QLabel( tr( "Temporary Speed Limits" ) );
-    label->setStyleSheet( "font: bold" );
+    label = new QLabel( tr( "Alternative Speed Limits" ) );
+    label->setStyleSheet( QString::fromAscii( "font: bold" ) );
     label->setAlignment( Qt::AlignLeft|Qt::AlignVCenter );
     h->addWidget( label );
     hig->addSectionTitle( h );
@@ -292,12 +245,12 @@ PrefsDialog :: createSpeedTab( )
         QString s = tr( "<small>Override normal speed limits manually or at scheduled times</small>" );
         hig->addWideControl( new QLabel( s ) );
 
-        s = tr( "Limit d&ownload speed (KB/s):" );
-        r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_DOWN, 0, INT_MAX, 5 );
+        s = tr( "U&pload (%1):" ).arg( speed_K_str );
+        r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_UP, 0, INT_MAX, 5 );
         hig->addRow( s, r );
 
-        s = tr( "Limit u&pload speed (KB/s):" );
-        r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_UP, 0, INT_MAX, 5 );
+        s = tr( "Do&wnload (%1):" ).arg( speed_K_str );
+        r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_DOWN, 0, INT_MAX, 5 );
         hig->addRow( s, r );
 
         QCheckBox * c = checkBoxNew( tr( "&Scheduled times:" ), Prefs::ALT_SPEED_LIMIT_TIME_ENABLED );
@@ -306,7 +259,7 @@ PrefsDialog :: createSpeedTab( )
         QWidget * w = timeEditNew( Prefs :: ALT_SPEED_LIMIT_TIME_BEGIN );
         h->addWidget( w, 1 );
         mySchedWidgets << w;
-        QLabel * nd = new QLabel( "&to" );
+        QLabel * nd = new QLabel( tr("&to") );
         h->addWidget( nd );
         mySchedWidgets << nd;
         w = timeEditNew( Prefs :: ALT_SPEED_LIMIT_TIME_END );
@@ -341,6 +294,30 @@ PrefsDialog :: createSpeedTab( )
 ****
 ***/
 
+QWidget *
+PrefsDialog :: createDesktopTab( )
+{
+    HIG * hig = new HIG( this );
+    hig->addSectionTitle( tr( "Desktop" ) );
+
+      hig->addWideControl( checkBoxNew( tr( "Show Transmission icon in the &notification area" ), Prefs::SHOW_TRAY_ICON ) );
+      hig->addWideControl( checkBoxNew( tr( "Start &minimized in notification area" ), Prefs::START_MINIMIZED ) );
+
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr ("Notification") );
+
+      hig->addWideControl( checkBoxNew( tr( "Show a notification when torrents are a&dded" ), Prefs::SHOW_NOTIFICATION_ON_ADD ) );
+      hig->addWideControl( checkBoxNew( tr( "Show a notification when torrents &finish" ), Prefs::SHOW_NOTIFICATION_ON_COMPLETE ) );
+      hig->addWideControl( checkBoxNew( tr( "Play a &sound when torrents finish" ), Prefs::COMPLETE_SOUND_ENABLED ) );
+
+    hig->finish( );
+    return hig;
+}
+
+/***
+****
+***/
+
 void
 PrefsDialog :: onPortTested( bool isOpen )
 {
@@ -353,7 +330,7 @@ PrefsDialog :: onPortTested( bool isOpen )
 void
 PrefsDialog :: onPortTest( )
 {
-    myPortLabel->setText( tr( "Testing..." ) );
+    myPortLabel->setText( tr( "Testing TCP Port..." ) );
     myPortButton->setEnabled( false );
     myWidgets[Prefs::PEER_PORT]->setEnabled( false );
     mySession.portTest( );
@@ -367,7 +344,7 @@ PrefsDialog :: createNetworkTab( )
 
     QSpinBox * s = spinBoxNew( Prefs::PEER_PORT, 1, 65535, 1 );
     QHBoxLayout * h = new QHBoxLayout( );
-    QPushButton * b = myPortButton = new QPushButton( tr( "&Test Port" ) );
+    QPushButton * b = myPortButton = new QPushButton( tr( "Te&st Port" ) );
     QLabel * l = myPortLabel = new QLabel( tr( "Status unknown" ) );
     h->addWidget( l );
     h->addSpacing( HIG :: PAD_BIG );
@@ -377,14 +354,27 @@ PrefsDialog :: createNetworkTab( )
     connect( &mySession, SIGNAL(portTested(bool)), this, SLOT(onPortTested(bool)));
 
     hig->addRow( tr( "&Port for incoming connections:" ), s );
-    hig->addRow( "", h, 0 );
-    hig->addWideControl( checkBoxNew( tr( "Use UPnP or NAT-PMP port &forwarding from my router" ), Prefs::PORT_FORWARDING ) );
+    hig->addRow( QString(), h, 0 );
     hig->addWideControl( checkBoxNew( tr( "Pick a &random port every time Transmission is started" ), Prefs :: PEER_PORT_RANDOM_ON_START ) );
+    hig->addWideControl( checkBoxNew( tr( "Use UPnP or NAT-PMP port &forwarding from my router" ), Prefs::PORT_FORWARDING ) );
 
     hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Limits" ) );
-    hig->addRow( tr( "Maximum peers per &torrent:" ), spinBoxNew( Prefs::PEER_LIMIT_TORRENT, 1, 300, 5 ) );
-    hig->addRow( tr( "Maximum peers &overall:" ), spinBoxNew( Prefs::PEER_LIMIT_GLOBAL, 1, 3000, 5 ) );
+    hig->addSectionTitle( tr( "Peer Limits" ) );
+    hig->addRow( tr( "Maximum peers per &torrent:" ), spinBoxNew( Prefs::PEER_LIMIT_TORRENT, 1, FD_SETSIZE, 5 ) );
+    hig->addRow( tr( "Maximum peers &overall:" ), spinBoxNew( Prefs::PEER_LIMIT_GLOBAL, 1, FD_SETSIZE, 5 ) );
+
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Options" ) );
+
+    QWidget * w;
+    hig->addWideControl( w = checkBoxNew( tr( "Enable &uTP for peer connections" ), Prefs::UTP_ENABLED ) );
+    w->setToolTip( tr( "uTP is a tool for reducing network congestion." ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Use PE&X to find more peers" ), Prefs::PEX_ENABLED ) );
+    w->setToolTip( tr( "PEX is a tool for exchanging peer lists with the peers you're connected to." ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Use &DHT to find more peers" ), Prefs::DHT_ENABLED ) );
+    w->setToolTip( tr( "DHT is a tool for finding peers without a tracker." ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Use &Local Peer Discovery to find more peers" ), Prefs::LPD_ENABLED ) );
+    w->setToolTip( tr( "LPD is a tool for finding peers on your local network." ) );
 
     hig->finish( );
     return hig;
@@ -420,47 +410,30 @@ void
 PrefsDialog :: onUpdateBlocklistClicked( )
 {
     myBlocklistDialog = new QMessageBox( QMessageBox::Information,
-                                         "",
+                                         QString(),
                                          tr( "<b>Update Blocklist</b><p>Getting new blocklist..." ),
                                          QMessageBox::Close,
                                          this );
-    QPixmap pixmap;
-    QIcon icon = QtIconLoader :: icon( "dialog-information" );
-    if( !icon.isNull( ) ) {
-        const int size = style()->pixelMetric( QStyle::PM_LargeIconSize );
-        myBlocklistDialog->setIconPixmap( icon.pixmap( size, size ) );
-    }
     connect( myBlocklistDialog, SIGNAL(rejected()), this, SLOT(onUpdateBlocklistCancelled()) );
     connect( &mySession, SIGNAL(blocklistUpdated(int)), this, SLOT(onBlocklistUpdated(int))) ;
     myBlocklistDialog->show( );
     mySession.updateBlocklist( );
 }
-    
+
 void
 PrefsDialog :: encryptionEdited( int i )
 {
     const int value( qobject_cast<QComboBox*>(sender())->itemData(i).toInt( ) );
-    myPrefs.set( Prefs::ENCRYPTION, value );
+    setPref( Prefs::ENCRYPTION, value );
 }
 
 QWidget *
 PrefsDialog :: createPrivacyTab( )
 {
+    QWidget * w;
     HIG * hig = new HIG( this );
-    hig->addSectionTitle( tr( "Blocklist" ) );
-    QHBoxLayout * h = new QHBoxLayout( );
-    QIcon i( style()->standardIcon( QStyle::StandardPixmap( QStyle::SP_BrowserReload ) ) );
-    QPushButton * w = new QPushButton( i, tr( "&Update blocklist" ) );
-    connect( w, SIGNAL(clicked(bool)), this, SLOT(onUpdateBlocklistClicked()));
-    myBlockWidgets << w;
-    QWidget * l = checkBoxNew( "", Prefs::BLOCKLIST_ENABLED );
-    h->addWidget( l );
-    h->addStretch( 1 );
-    h->addWidget( w );
-    hig->addWideControl( h );
-    l = checkBoxNew( tr( "Enable &automatic updates" ), Prefs::BLOCKLIST_UPDATES_ENABLED );
-    myBlockWidgets << l;
-    hig->addWideControl( l );
+
+    hig->addSectionTitle( tr( "Encryption" ) );
 
     QComboBox * box = new QComboBox( );
     box->addItem( tr( "Allow encryption" ), 0 );
@@ -469,14 +442,33 @@ PrefsDialog :: createPrivacyTab( )
     myWidgets.insert( Prefs :: ENCRYPTION, box );
     connect( box, SIGNAL(activated(int)), this, SLOT(encryptionEdited(int)));
 
-    hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Privacy" ) );
     hig->addRow( tr( "&Encryption mode:" ), box );
-    hig->addWideControl( checkBoxNew( tr( "Use PE&X to find more peers" ), Prefs::PEX_ENABLED ) );
-    hig->addWideControl( checkBoxNew( tr( "Use &DHT to find more peers" ), Prefs::DHT_ENABLED ) );
+
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Blocklist" ) );
+
+    QWidget * l = checkBoxNew( tr("Enable &blocklist:"), Prefs::BLOCKLIST_ENABLED );
+    QWidget * e = lineEditNew( Prefs::BLOCKLIST_URL );
+    myBlockWidgets << e;
+    hig->addRow( l, e );
+
+    l = myBlocklistLabel = new QLabel( );
+    myBlockWidgets << l;
+    w = new QPushButton( tr( "&Update" ) );
+    connect( w, SIGNAL(clicked(bool)), this, SLOT(onUpdateBlocklistClicked()));
+    myBlockWidgets << w;
+    QHBoxLayout * h = new QHBoxLayout( );
+    h->addWidget( l );
+    h->addStretch( 1 );
+    h->addWidget( w );
+    hig->addWideControl( h );
+
+    l = checkBoxNew( tr( "Enable &automatic updates" ), Prefs::BLOCKLIST_UPDATES_ENABLED );
+    myBlockWidgets << l;
+    hig->addWideControl( l );
 
     hig->finish( );
-    updateBlocklistCheckBox( );
+    updateBlocklistLabel( );
     return hig;
 }
 
@@ -485,77 +477,151 @@ PrefsDialog :: createPrivacyTab( )
 ***/
 
 void
-PrefsDialog :: onWatchClicked( void )
+PrefsDialog :: onScriptClicked( void )
 {
-    QFileDialog * d = new QFileDialog( this,
-                                       tr( "Select Watch Directory" ),
-                                       myPrefs.getString( Prefs::DIR_WATCH ) );
-    d->setFileMode( QFileDialog::Directory );
-    connect( d, SIGNAL(filesSelected(const QStringList&)), this, SLOT(onWatchSelected(const QStringList&)) );
-    d->show( );
+    const QString title = tr( "Select \"Torrent Done\" Script" );
+    const QString myPath = myPrefs.getString( Prefs::SCRIPT_TORRENT_DONE_FILENAME );
+    const QString path = Utils::remoteFileChooser( this, title, myPath, false, mySession.isServer() );
+
+    if( !path.isEmpty() )
+        onLocationSelected( path, Prefs::SCRIPT_TORRENT_DONE_FILENAME );
 }
 
 void
-PrefsDialog :: onWatchSelected( const QStringList& list )
+PrefsDialog :: onIncompleteClicked( void )
 {
-    if( list.size() == 1 )
-        myPrefs.set( Prefs::DIR_WATCH, list.first( ) );
+    const QString title = tr( "Select Incomplete Directory" );
+    const QString myPath = myPrefs.getString( Prefs::INCOMPLETE_DIR );
+    const QString path = Utils::remoteFileChooser( this, title, myPath, true, mySession.isServer() );
+
+    if( !path.isEmpty() )
+        onLocationSelected( path, Prefs::INCOMPLETE_DIR );
+}
+
+void
+PrefsDialog :: onWatchClicked( void )
+{
+    const QString title = tr( "Select Watch Directory" );
+    const QString myPath = myPrefs.getString( Prefs::DIR_WATCH );
+    const QString path = Utils::remoteFileChooser( this, title, myPath, true, true );
+
+    if( !path.isEmpty() )
+        onLocationSelected( path, Prefs::DIR_WATCH );
 }
 
 void
 PrefsDialog :: onDestinationClicked( void )
 {
-    QFileDialog * d = new QFileDialog( this,
-                                       tr( "Select Destination" ),
-                                       myPrefs.getString( Prefs::DOWNLOAD_DIR ) );
-    d->setFileMode( QFileDialog::Directory );
-    connect( d, SIGNAL(filesSelected(const QStringList&)), this, SLOT(onDestinationSelected(const QStringList&)) );
-    d->show( );
+    const QString title = tr( "Select Destination" );
+    const QString myPath = myPrefs.getString( Prefs::DOWNLOAD_DIR );
+    const QString path = Utils::remoteFileChooser( this, title, myPath, true, mySession.isServer() );
+
+    if( !path.isEmpty() )
+        onLocationSelected( path, Prefs::DOWNLOAD_DIR );
 }
 
 void
-PrefsDialog :: onDestinationSelected( const QStringList& list )
+PrefsDialog :: onLocationSelected( const QString& path, int key )
 {
-    if( list.size() == 1 )
-        myPrefs.set( Prefs::DOWNLOAD_DIR, list.first( ) );
+    setPref( key, path );
 }
 
 QWidget *
-PrefsDialog :: createTorrentsTab( )
+PrefsDialog :: createSeedingTab( )
 {
     const int iconSize( style( )->pixelMetric( QStyle :: PM_SmallIconSize ) );
-    const QIcon folderIcon = QtIconLoader :: icon( "folder", style()->standardIcon( QStyle::SP_DirIcon ) );
+    const QFileIconProvider iconProvider;
+    const QIcon folderIcon = iconProvider.icon( QFileIconProvider::Folder );
     const QPixmap folderPixmap = folderIcon.pixmap( iconSize );
+    const QIcon fileIcon = iconProvider.icon( QFileIconProvider::File );
+    const QPixmap filePixmap = fileIcon.pixmap( iconSize );
 
     QWidget *l, *r;
     HIG * hig = new HIG( this );
-    hig->addSectionTitle( tr( "Adding Torrents" ) );
+    hig->addSectionTitle( tr( "Limits" ) );
 
-        l = checkBoxNew( tr( "Automatically &add torrents from:" ), Prefs::DIR_WATCH_ENABLED );
-        QPushButton * b = myWatchButton = new QPushButton;
+        l = checkBoxNew( tr( "Stop seeding at &ratio:" ), Prefs::RATIO_ENABLED );
+        r = doubleSpinBoxNew( Prefs::RATIO, 0, INT_MAX, 0.5, 2 );
+        hig->addRow( l, r );
+        enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
+
+        l = checkBoxNew( tr( "Stop seeding if idle for &N minutes:" ), Prefs::IDLE_LIMIT_ENABLED );
+        r = spinBoxNew( Prefs::IDLE_LIMIT, 1, INT_MAX, 5 );
+        hig->addRow( l, r );
+        enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
+
+    hig->finish( );
+    return hig;
+}
+
+QWidget *
+PrefsDialog :: createDownloadingTab( )
+{
+    const int iconSize( style( )->pixelMetric( QStyle :: PM_SmallIconSize ) );
+    const QFileIconProvider iconProvider;
+    const QIcon folderIcon = iconProvider.icon( QFileIconProvider::Folder );
+    const QPixmap folderPixmap = folderIcon.pixmap( iconSize );
+    const QIcon fileIcon = iconProvider.icon( QFileIconProvider::File );
+    const QPixmap filePixmap = fileIcon.pixmap( iconSize );
+
+    QWidget * l;
+    QPushButton * b;
+    HIG * hig = new HIG( this );
+    hig->addSectionTitle( tr( "Adding" ) );
+
+        l = checkBoxNew( tr( "Automatically add .torrent files &from:" ), Prefs::DIR_WATCH_ENABLED );
+        b = myWatchButton = new QPushButton;
         b->setIcon( folderPixmap );
-        b->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        b->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
         connect( b, SIGNAL(clicked(bool)), this, SLOT(onWatchClicked(void)) );
         hig->addRow( l, b );
         enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), b );
 
-        hig->addWideControl( checkBoxNew( tr( "Display &options dialog" ), Prefs::OPTIONS_PROMPT ) );
-        hig->addWideControl( checkBoxNew( tr( "&Start when added" ), Prefs::START ) );
-        hig->addWideControl( checkBoxNew( tr( "&Delete source files" ), Prefs::TRASH_ORIGINAL ) );
+        hig->addWideControl( checkBoxNew( tr( "Show the Torrent Options &dialog" ), Prefs::OPTIONS_PROMPT ) );
+
+        hig->addWideControl( checkBoxNew( tr( "&Start added torrents" ), Prefs::START ) );
+
+        hig->addWideControl( checkBoxNew( tr( "Mo&ve the .torrent file to the trash" ), Prefs::TRASH_ORIGINAL ) );
 
         b = myDestinationButton = new QPushButton;
         b->setIcon( folderPixmap );
-        b->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        b->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
         connect( b, SIGNAL(clicked(bool)), this, SLOT(onDestinationClicked(void)) );
         hig->addRow( tr( "Save to &Location:" ), b );
 
-    hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Limits" ) );
+        const QString downloadDir (myPrefs.getString(Prefs::DOWNLOAD_DIR));
+        l = myFreespaceLabel = new FreespaceLabel (mySession, downloadDir, this);
+        QHBoxLayout * h = new QHBoxLayout ();
+        h->addStretch (1);
+        h->addWidget (l);
+        hig->addWideControl (h);
 
-        l = checkBoxNew( tr( "&Seed torrent until its ratio reaches:" ), Prefs::RATIO_ENABLED );
-        r = doubleSpinBoxNew( Prefs::RATIO, 0, INT_MAX, 0.5, 2 );
-        hig->addRow( l, r );
-        enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Download Queue" ) );
+    
+        hig->addRow( tr( "Ma&ximum active downloads:" ), spinBoxNew( Prefs::DOWNLOAD_QUEUE_SIZE, 1, INT_MAX, 1 ) );
+        hig->addRow( tr( "Downloads sharing data in the last &N minutes are active:" ), spinBoxNew( Prefs::QUEUE_STALLED_MINUTES, 1, INT_MAX, 10 ) );
+
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Incomplete" ) );
+
+        hig->addWideControl( checkBoxNew( tr( "Append \".&part\" to incomplete files' names" ), Prefs::RENAME_PARTIAL_FILES ) );
+
+        l = myIncompleteCheckbox = checkBoxNew( tr( "Keep &incomplete files in:" ), Prefs::INCOMPLETE_DIR_ENABLED );
+        b = myIncompleteButton = new QPushButton;
+        b->setIcon( folderPixmap );
+        b->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
+        connect( b, SIGNAL(clicked(bool)), this, SLOT(onIncompleteClicked(void)) );
+        hig->addRow( myIncompleteCheckbox, b );
+        enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), b );
+
+        l = myTorrentDoneScriptCheckbox = checkBoxNew( tr( "Call scrip&t when torrent is completed:" ), Prefs::SCRIPT_TORRENT_DONE_ENABLED );
+        b = myTorrentDoneScriptButton = new QPushButton;
+        b->setIcon( filePixmap );
+        b->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
+        connect( b, SIGNAL(clicked(bool)), this, SLOT(onScriptClicked(void)) );
+        hig->addRow( myTorrentDoneScriptCheckbox, b );
+        enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), b );
 
     hig->finish( );
     return hig;
@@ -575,32 +641,35 @@ PrefsDialog :: PrefsDialog( Session& session, Prefs& prefs, QWidget * parent ):
     setWindowTitle( tr( "Transmission Preferences" ) );
 
     QTabWidget * t = new QTabWidget( this );
-    t->addTab( createTorrentsTab( ),     tr( "Torrents" ) );
     t->addTab( createSpeedTab( ),        tr( "Speed" ) );
+    t->addTab( createDownloadingTab( ),  tr( "Downloading" ) );
+    t->addTab( createSeedingTab( ),      tr( "Seeding" ) );
     t->addTab( createPrivacyTab( ),      tr( "Privacy" ) );
     t->addTab( createNetworkTab( ),      tr( "Network" ) );
-    t->addTab( createWebTab( session ),  tr( "Web" ) );
-    //t->addTab( createTrackerTab( ),    tr( "Trackers" ) );
+    t->addTab( createDesktopTab( ),      tr( "Desktop" ) );
+    t->addTab( createRemoteTab(session), tr( "Remote" ) );
     myLayout->addWidget( t );
 
     QDialogButtonBox * buttons = new QDialogButtonBox( QDialogButtonBox::Close, Qt::Horizontal, this );
-    connect( buttons, SIGNAL(rejected()), this, SLOT(hide()) ); // "close" triggers rejected
+    connect( buttons, SIGNAL(rejected()), this, SLOT(close()) ); // "close" triggers rejected
     myLayout->addWidget( buttons );
+    QWidget::setAttribute( Qt::WA_DeleteOnClose, true );
 
-    connect( &myPrefs, SIGNAL(changed(int)), this, SLOT(updatePref(int)));
     connect( &mySession, SIGNAL(sessionUpdated()), this, SLOT(sessionUpdated()));
 
     QList<int> keys;
     keys << Prefs :: RPC_ENABLED
-         << Prefs :: PROXY_ENABLED
          << Prefs :: ALT_SPEED_LIMIT_ENABLED
          << Prefs :: ALT_SPEED_LIMIT_TIME_ENABLED
          << Prefs :: ENCRYPTION
          << Prefs :: BLOCKLIST_ENABLED
          << Prefs :: DIR_WATCH
-         << Prefs :: DOWNLOAD_DIR;
+         << Prefs :: DOWNLOAD_DIR
+         << Prefs :: INCOMPLETE_DIR
+         << Prefs :: INCOMPLETE_DIR_ENABLED
+         << Prefs :: SCRIPT_TORRENT_DONE_FILENAME;
     foreach( int key, keys )
-        updatePref( key );
+        refreshPref( key );
 
     // if it's a remote session, disable the preferences
     // that don't work in remote sessions
@@ -616,6 +685,13 @@ PrefsDialog :: ~PrefsDialog( )
 {
 }
 
+void
+PrefsDialog :: setPref( int key, const QVariant& v )
+{
+    myPrefs.set( key, v );
+    refreshPref( key );
+}
+
 /***
 ****
 ***/
@@ -623,22 +699,18 @@ PrefsDialog :: ~PrefsDialog( )
 void
 PrefsDialog :: sessionUpdated( )
 {
-    updateBlocklistCheckBox( );
+    updateBlocklistLabel( );
 }
 
 void
-PrefsDialog :: updateBlocklistCheckBox( )
+PrefsDialog :: updateBlocklistLabel( )
 {
-    QCheckBox * box = qobject_cast<QCheckBox*>( myWidgets[Prefs::BLOCKLIST_ENABLED] );
     const int n = mySession.blocklistSize( );
-    if( n < 0 ) // unknown
-        box->setText( tr( "Enable &blocklist" ) );
-    else
-        box->setText( tr( "Enable &blocklist (%Ln rules)", 0, n ) );
+    myBlocklistLabel->setText( tr( "<i>Blocklist contains %Ln rules</i>", 0, n ) );
 }
 
 void
-PrefsDialog :: updatePref( int key )
+PrefsDialog :: refreshPref( int key )
 {
     switch( key )
     {
@@ -648,21 +720,12 @@ PrefsDialog :: updatePref( int key )
             const bool enabled( myPrefs.getBool( Prefs::RPC_ENABLED ) );
             const bool whitelist( myPrefs.getBool( Prefs::RPC_WHITELIST_ENABLED ) );
             const bool auth( myPrefs.getBool( Prefs::RPC_AUTH_REQUIRED ) );
-            foreach( QWidget * w, myWebWhitelistWidgets ) w->setEnabled( enabled && whitelist ); 
-            foreach( QWidget * w, myWebAuthWidgets ) w->setEnabled( enabled && auth ); 
+            foreach( QWidget * w, myWebWhitelistWidgets ) w->setEnabled( enabled && whitelist );
+            foreach( QWidget * w, myWebAuthWidgets ) w->setEnabled( enabled && auth );
             foreach( QWidget * w, myWebWidgets ) w->setEnabled( enabled );
             break;
         }
 
-        case Prefs :: PROXY_ENABLED:
-        case Prefs :: PROXY_AUTH_ENABLED: {
-            const bool enabled( myPrefs.getBool( Prefs::PROXY_ENABLED ) );
-            const bool auth( myPrefs.getBool( Prefs::PROXY_AUTH_ENABLED ) );
-            foreach( QWidget * w, myProxyAuthWidgets ) w->setEnabled( enabled && auth ); 
-            foreach( QWidget * w, myProxyWidgets ) w->setEnabled( enabled );
-            break;
-        }
-
         case Prefs :: ALT_SPEED_LIMIT_TIME_ENABLED: {
             const bool enabled = myPrefs.getBool( key );
             foreach( QWidget * w, mySchedWidgets ) w->setEnabled( enabled );
@@ -679,14 +742,33 @@ PrefsDialog :: updatePref( int key )
             myWatchButton->setText( QFileInfo(myPrefs.getString(Prefs::DIR_WATCH)).fileName() );
             break;
 
+        case Prefs :: SCRIPT_TORRENT_DONE_FILENAME: {
+            const QString path( myPrefs.getString( key ) );
+            myTorrentDoneScriptButton->setText( QFileInfo(path).fileName() );
+            break;
+        }
+
         case Prefs :: PEER_PORT:
             myPortLabel->setText( tr( "Status unknown" ) );
             myPortButton->setEnabled( true );
             break;
 
         case Prefs :: DOWNLOAD_DIR: {
-            QString path( myPrefs.getString( Prefs :: DOWNLOAD_DIR ) );
+            const QString path( myPrefs.getString( key ) );
             myDestinationButton->setText( QFileInfo(path).fileName() );
+            myFreespaceLabel->setPath (path);
+            break;
+        }
+
+        case Prefs :: INCOMPLETE_DIR: {
+            QString path( myPrefs.getString( key ) );
+            myIncompleteButton->setText( QFileInfo(path).fileName() );
+            break;
+        }
+
+        case Prefs :: INCOMPLETE_DIR_ENABLED: {
+            const bool enabled = myPrefs.getBool( key );
+            myIncompleteButton->setEnabled( enabled );
             break;
         }
 
